#!/usr/bin/env python3
"""
Price Alarm Manager

Manages price alarms for tokens with persistence and monitoring integration.
"""

import json
import os
import logging
from datetime import datetime
from typing import Dict, List, Any, Optional

logger = logging.getLogger(__name__)

class AlarmManager:
    """Manages price alarms with persistence and monitoring."""
    
    def __init__(self, alarms_file: str = "data/price_alarms.json"):
        """Initialize the alarm manager."""
        self.alarms_file = alarms_file
        
        # Ensure the data directory exists
        data_dir = os.path.dirname(self.alarms_file)
        if data_dir and not os.path.exists(data_dir):
            os.makedirs(data_dir)
            logger.info(f"Created data directory: {data_dir}")
            
        self.data = self._load_alarms()
        
        # Initialize if first run
        if not self.data:
            self._initialize_alarms()
    
    def _load_alarms(self) -> Dict[str, Any]:
        """Load alarms from file."""
        try:
            if os.path.exists(self.alarms_file):
                with open(self.alarms_file, 'r') as f:
                    loaded_data = json.load(f)
                    # Basic validation
                    if isinstance(loaded_data, dict) and 'next_id' in loaded_data and 'alarms' in loaded_data:
                        return loaded_data
                    else:
                        logger.warning(f"Alarms file {self.alarms_file} has invalid format. Re-initializing.")
                        return {}
        except json.JSONDecodeError as e:
            logger.error(f"Error decoding JSON from {self.alarms_file}: {e}. Re-initializing.")
            return {}
        except Exception as e:
            logger.error(f"Error loading alarms from {self.alarms_file}: {e}")
        return {}
    
    def _save_alarms(self):
        """Save alarms to file."""
        try:
            with open(self.alarms_file, 'w') as f:
                json.dump(self.data, f, indent=2, default=str)
        except Exception as e:
            logger.error(f"Error saving alarms to {self.alarms_file}: {e}")
    
    def _initialize_alarms(self):
        """Initialize alarms structure."""
        self.data = {
            'next_id': 1,
            'alarms': [],
            'triggered_alarms': [],
            'last_update': datetime.now().isoformat()
        }
        self._save_alarms()
    
    def create_alarm(self, token: str, target_price: float, current_price: float) -> Dict[str, Any]:
        """Create a new price alarm."""
        direction = 'above' if target_price > current_price else 'below'
        
        alarm = {
            'id': self.data['next_id'],
            'token': token.upper(),
            'target_price': target_price,
            'current_price_at_creation': current_price,
            'direction': direction,
            'status': 'active',
            'created_at': datetime.now().isoformat(),
            'triggered_at': None,
            'triggered_price': None
        }
        
        self.data['alarms'].append(alarm)
        self.data['next_id'] += 1
        self.data['last_update'] = datetime.now().isoformat()
        self._save_alarms()
        
        logger.info(f"Created alarm {alarm['id']}: {token} {direction} ${target_price}")
        return alarm
    
    def get_alarm_by_id(self, alarm_id: int) -> Optional[Dict[str, Any]]:
        """Get alarm by ID."""
        for alarm in self.data.get('alarms', []):
            if alarm['id'] == alarm_id:
                return alarm
        return None
    
    def get_alarms_by_token(self, token: str) -> List[Dict[str, Any]]:
        """Get all active alarms for a specific token."""
        token = token.upper()
        return [alarm for alarm in self.data.get('alarms', []) if alarm['token'] == token and alarm['status'] == 'active']
    
    def get_all_active_alarms(self) -> List[Dict[str, Any]]:
        """Get all active alarms."""
        return [alarm for alarm in self.data.get('alarms', []) if alarm['status'] == 'active']
    
    def get_recent_triggered_alarms(self, limit: int = 10) -> List[Dict[str, Any]]:
        """Get recently triggered alarms."""
        triggered = self.data.get('triggered_alarms', [])
        return triggered[-limit:]
    
    def remove_alarm(self, alarm_id: int) -> bool:
        """Remove an alarm by ID."""
        alarms_list = self.data.get('alarms', [])
        for i, alarm in enumerate(alarms_list):
            if alarm['id'] == alarm_id:
                removed_alarm = alarms_list.pop(i)
                self.data['last_update'] = datetime.now().isoformat()
                self._save_alarms()
                logger.info(f"Removed alarm {alarm_id}: {removed_alarm['token']} @ ${removed_alarm['target_price']}")
                return True
        return False
    
    def trigger_alarm(self, alarm_id: int, triggered_price: float) -> Optional[Dict[str, Any]]:
        """Mark an alarm as triggered."""
        alarms_list = self.data.get('alarms', [])
        for alarm in alarms_list:
            if alarm['id'] == alarm_id and alarm['status'] == 'active':
                alarm['status'] = 'triggered'
                alarm['triggered_at'] = datetime.now().isoformat()
                alarm['triggered_price'] = triggered_price
                
                triggered_alarms_list = self.data.get('triggered_alarms', [])
                triggered_alarms_list.append(alarm.copy())
                self.data['triggered_alarms'] = triggered_alarms_list[-50:] # Keep only last 50
                
                self.data['last_update'] = datetime.now().isoformat()
                self._save_alarms()
                
                logger.info(f"Triggered alarm {alarm_id}: {alarm['token']} @ ${triggered_price}")
                return alarm
        return None
    
    def check_alarms(self, price_data: Dict[str, float]) -> List[Dict[str, Any]]:
        """Check all active alarms against current prices and trigger if needed."""
        triggered_alarms = []
        for alarm in self.get_all_active_alarms():
            token = alarm['token']
            if token in price_data:
                current_price = price_data[token]
                target_price = alarm['target_price']
                direction = alarm['direction']
                
                if (direction == 'above' and current_price >= target_price) or \
                   (direction == 'below' and current_price <= target_price):
                    triggered_alarm_obj = self.trigger_alarm(alarm['id'], current_price)
                    if triggered_alarm_obj:
                        triggered_alarms.append(triggered_alarm_obj)
        return triggered_alarms
    
    def get_statistics(self) -> Dict[str, Any]:
        """Get alarm statistics."""
        active_alarms = self.get_all_active_alarms()
        triggered_alarms = self.data.get('triggered_alarms', [])
        
        token_counts = {}
        for alarm in active_alarms:
            token_counts[alarm['token']] = token_counts.get(alarm['token'], 0) + 1
        
        return {
            'total_active': len(active_alarms),
            'total_triggered': len(triggered_alarms),
            'tokens_tracked': len(token_counts),
            'token_breakdown': token_counts,
            'next_id': self.data.get('next_id', 1)
        }
    
    def format_alarm_list(self, alarms: List[Dict[str, Any]], title: str = "Active Price Alarms") -> str:
        """Format a list of alarms for display."""
        if not alarms:
            return f"🔔 <b>{title}</b>\n\n📭 No active alarms.\nUse <code>/alarm TOKEN PRICE</code> to set one."
        
        message = f"🔔 <b>{title}</b>\n\n"
        for alarm in alarms:
            direction_emoji = "📈" if alarm['direction'] == 'above' else "📉"
            created_price = alarm.get('current_price_at_creation', alarm['target_price'])
            price_diff = abs(alarm['target_price'] - created_price)
            price_diff_percent = (price_diff / created_price * 100) if created_price > 0 else 0
            
            message += f"• ID {alarm['id']}: {alarm['token']} {direction_emoji} ${alarm['target_price']:,.2f}\n"
            message += f"  (Created at: ${created_price:,.2f}, Diff: {price_diff_percent:.1f}%)\n"
        
        message += "\n💡 Use <code>/alarm ID</code> to remove an alarm."
        return message.strip()
    
    def format_triggered_alarm(self, alarm: Dict[str, Any]) -> str:
        """Format a triggered alarm notification."""
        direction_emoji = "📈" if alarm['direction'] == 'above' else "📉"
        triggered_at_str = "Unknown time"
        if alarm.get('triggered_at'):
            try:
                triggered_at_str = datetime.fromisoformat(alarm['triggered_at']).strftime('%H:%M:%S')
            except ValueError:
                pass # Keep as Unknown time

        message = f"""
🚨 <b>Price Alarm Triggered!</b>

🔔 <b>Alarm ID: {alarm['id']}</b>
• Token: {alarm['token']}
• Condition: Price {alarm['direction']} ${alarm['target_price']:,.2f}
• Triggered Price: ${alarm.get('triggered_price', 0.0):,.2f} {direction_emoji}

⏰ <b>Time:</b> {triggered_at_str}
💡 This alarm is now inactive.
        """
        return message.strip() 