#!/usr/bin/env python3
"""
Market Monitor - Handles external trade monitoring and heartbeat functionality.
"""

import logging
import asyncio
from datetime import datetime, timedelta
from typing import Optional, Dict, Any, List

from src.config.config import Config
from src.monitoring.alarm_manager import AlarmManager

logger = logging.getLogger(__name__)

class MarketMonitor:
    """Handles external trade monitoring and market events."""
    
    def __init__(self, trading_engine):
        """Initialize the market monitor."""
        self.trading_engine = trading_engine
        self.is_running = False
        self._monitor_task = None
        
        # External trade monitoring
        self.last_processed_trade_time = None
        
        # Alarm management
        self.alarm_manager = AlarmManager()
        
        # Order monitoring
        self.last_known_orders = set()
        self.last_known_positions = {}
        
        # Notification manager (will be set by the core bot)
        self.notification_manager = None
        
    def set_notification_manager(self, notification_manager):
        """Set the notification manager for sending alerts."""
        self.notification_manager = notification_manager
        
    async def start(self):
        """Start the market monitor."""
        if self.is_running:
            return
        
        self.is_running = True
        logger.info("🔄 Market monitor started")
        
        # Initialize tracking
        await self._initialize_tracking()
        
        # Start monitoring task
        self._monitor_task = asyncio.create_task(self._monitor_loop())
    
    async def stop(self):
        """Stop the market monitor."""
        if not self.is_running:
            return
        
        self.is_running = False
        
        if self._monitor_task:
            self._monitor_task.cancel()
            try:
                await self._monitor_task
            except asyncio.CancelledError:
                pass
        
        logger.info("🛑 Market monitor stopped")
    
    async def _initialize_tracking(self):
        """Initialize order and position tracking."""
        try:
            # Get current open orders to initialize tracking
            orders = self.trading_engine.get_orders()
            if orders:
                self.last_known_orders = {order.get('id') for order in orders if order.get('id')}
                logger.info(f"📋 Initialized tracking with {len(self.last_known_orders)} open orders")
            
            # Get current positions for P&L tracking
            positions = self.trading_engine.get_positions()
            if positions:
                for position in positions:
                    symbol = position.get('symbol')
                    contracts = float(position.get('contracts', 0))
                    entry_price = float(position.get('entryPx', 0))
                    
                    if symbol and contracts != 0:
                        self.last_known_positions[symbol] = {
                            'contracts': contracts,
                            'entry_price': entry_price
                        }
                logger.info(f"📊 Initialized tracking with {len(self.last_known_positions)} positions")
                
        except Exception as e:
            logger.error(f"❌ Error initializing tracking: {e}")
    
    async def _monitor_loop(self):
        """Main monitoring loop that runs every BOT_HEARTBEAT_SECONDS."""
        try:
            while self.is_running:
                await self._check_order_fills()
                await self._check_price_alarms()
                await self._check_external_trades()
                await asyncio.sleep(Config.BOT_HEARTBEAT_SECONDS)
        except asyncio.CancelledError:
            logger.info("Market monitor loop cancelled")
            raise
        except Exception as e:
            logger.error(f"Error in market monitor loop: {e}")
            # Restart after error
            if self.is_running:
                await asyncio.sleep(5)
                await self._monitor_loop()
    
    async def _check_order_fills(self):
        """Check for filled orders and send notifications."""
        try:
            # Get current orders and positions
            current_orders = self.trading_engine.get_orders() or []
            current_positions = self.trading_engine.get_positions() or []
            
            # Get current order IDs
            current_order_ids = {order.get('id') for order in current_orders if order.get('id')}
            
            # Find filled orders (orders that were in last_known_orders but not in current_orders)
            filled_order_ids = self.last_known_orders - current_order_ids
            
            if filled_order_ids:
                logger.info(f"🎯 Detected {len(filled_order_ids)} filled orders: {list(filled_order_ids)}")
                await self._process_filled_orders(filled_order_ids, current_positions)
            
            # Update tracking data
            self.last_known_orders = current_order_ids
            await self._update_position_tracking(current_positions)
            
        except Exception as e:
            logger.error(f"❌ Error checking order fills: {e}")
    
    async def _process_filled_orders(self, filled_order_ids: set, current_positions: list):
        """Process filled orders using enhanced position tracking."""
        try:
            # For bot-initiated orders, we'll detect changes in position size
            # and send appropriate notifications using the enhanced system
            
            # This method will be triggered when orders placed through the bot are filled
            # The external trade monitoring will handle trades made outside the bot
            
            # Update position tracking based on current positions
            await self._update_position_tracking(current_positions)
                
        except Exception as e:
            logger.error(f"❌ Error processing filled orders: {e}")
    
    async def _update_position_tracking(self, current_positions: list):
        """Update position tracking and calculate P&L changes."""
        try:
            new_position_map = {}
            
            for position in current_positions:
                symbol = position.get('symbol')
                contracts = float(position.get('contracts', 0))
                entry_price = float(position.get('entryPx', 0))
                
                if symbol and contracts != 0:
                    new_position_map[symbol] = {
                        'contracts': contracts,
                        'entry_price': entry_price
                    }
            
            # Compare with previous positions to detect changes
            for symbol, new_data in new_position_map.items():
                old_data = self.last_known_positions.get(symbol)
                
                if not old_data:
                    # New position opened
                    logger.info(f"📈 New position detected: {symbol} {new_data['contracts']} @ ${new_data['entry_price']}")
                elif abs(new_data['contracts'] - old_data['contracts']) > 0.000001:
                    # Position size changed
                    change = new_data['contracts'] - old_data['contracts']
                    logger.info(f"📊 Position change detected: {symbol} {change:+.6f} contracts")
            
            # Check for closed positions
            for symbol in self.last_known_positions:
                if symbol not in new_position_map:
                    logger.info(f"📉 Position closed: {symbol}")
            
            # Update tracking
            self.last_known_positions = new_position_map
            
        except Exception as e:
            logger.error(f"❌ Error updating position tracking: {e}")
    
    async def _check_price_alarms(self):
        """Check price alarms and trigger notifications."""
        try:
            active_alarms = self.alarm_manager.get_all_active_alarms()
            
            if not active_alarms:
                return
            
            # Group alarms by token to minimize API calls
            tokens_to_check = list(set(alarm['token'] for alarm in active_alarms))
            
            for token in tokens_to_check:
                try:
                    # Get current market price
                    symbol = f"{token}/USDC:USDC"
                    market_data = self.trading_engine.get_market_data(symbol)
                    
                    if not market_data or not market_data.get('ticker'):
                        continue
                    
                    current_price = float(market_data['ticker'].get('last', 0))
                    if current_price <= 0:
                        continue
                    
                    # Check alarms for this token
                    token_alarms = [alarm for alarm in active_alarms if alarm['token'] == token]
                    
                    for alarm in token_alarms:
                        target_price = alarm['target_price']
                        direction = alarm['direction']
                        
                        # Check if alarm should trigger
                        should_trigger = False
                        if direction == 'above' and current_price >= target_price:
                            should_trigger = True
                        elif direction == 'below' and current_price <= target_price:
                            should_trigger = True
                        
                        if should_trigger:
                            # Trigger the alarm
                            triggered_alarm = self.alarm_manager.trigger_alarm(alarm['id'], current_price)
                            if triggered_alarm:
                                await self._send_alarm_notification(triggered_alarm)
                
                except Exception as e:
                    logger.error(f"Error checking alarms for {token}: {e}")
                    
        except Exception as e:
            logger.error(f"❌ Error checking price alarms: {e}")
    
    async def _send_alarm_notification(self, alarm: Dict[str, Any]):
        """Send notification for triggered alarm."""
        try:
            # Send through notification manager if available
            if self.notification_manager:
                await self.notification_manager.send_alarm_triggered_notification(
                    alarm['token'], 
                    alarm['target_price'], 
                    alarm['triggered_price'], 
                    alarm['direction']
                )
            else:
                # Fallback to logging if notification manager not available
                logger.info(f"🔔 ALARM TRIGGERED: {alarm['token']} @ ${alarm['triggered_price']:,.2f}")
            
        except Exception as e:
            logger.error(f"❌ Error sending alarm notification: {e}")
    
    async def _check_external_trades(self):
        """Check for trades made outside the Telegram bot and update stats."""
        try:
            # Get recent fills from Hyperliquid
            recent_fills = self.trading_engine.client.get_recent_fills()
            
            if not recent_fills:
                return
            
            # Initialize last processed time if first run
            if self.last_processed_trade_time is None:
                # Set to current time minus 1 hour to catch recent activity
                self.last_processed_trade_time = datetime.now() - timedelta(hours=1)
            
            # Filter for new trades since last check
            new_trades = []
            latest_trade_time = self.last_processed_trade_time
            
            for fill in recent_fills:
                fill_time = fill.get('timestamp')
                if fill_time:
                    # Convert timestamps to comparable format
                    try:
                        # Convert fill_time to datetime object for comparison
                        if isinstance(fill_time, (int, float)):
                            # Assume it's a unix timestamp
                            fill_datetime = datetime.fromtimestamp(fill_time / 1000 if fill_time > 1e10 else fill_time)
                        else:
                            # Try to parse as ISO string
                            fill_datetime = datetime.fromisoformat(str(fill_time).replace('Z', '+00:00'))
                        
                        # Compare datetime objects
                        if fill_datetime > self.last_processed_trade_time:
                            new_trades.append(fill)
                            if fill_datetime > latest_trade_time:
                                latest_trade_time = fill_datetime
                    except Exception as timestamp_error:
                        logger.warning(f"⚠️ Error processing timestamp {fill_time}: {timestamp_error}")
                        continue
            
            if not new_trades:
                return
            
            # Process new trades
            for trade in new_trades:
                # Log trade processing for debugging
                trade_id = trade.get('id', 'external')
                symbol = trade.get('symbol', 'Unknown')
                side = trade.get('side', 'Unknown')
                amount = trade.get('amount', 0)
                price = trade.get('price', 0)
                
                logger.info(f"🔍 Processing external trade: {trade_id} - {side} {amount} {symbol} @ ${price}")
                
                await self._process_external_trade(trade)
            
            # Update last processed time (keep as datetime object)
            self.last_processed_trade_time = latest_trade_time
            
            if new_trades:
                logger.info(f"📊 Processed {len(new_trades)} external trades")
                
        except Exception as e:
            logger.error(f"❌ Error checking external trades: {e}")
    
    async def _process_external_trade(self, trade: Dict[str, Any]):
        """Process an individual external trade and determine if it's opening or closing a position."""
        try:
            # Extract trade information
            symbol = trade.get('symbol', '')
            side = trade.get('side', '')
            amount = float(trade.get('amount', 0))
            price = float(trade.get('price', 0))
            trade_id = trade.get('id', 'external')
            timestamp = trade.get('timestamp', '')
            
            if not all([symbol, side, amount, price]):
                return
            
            # Skip bot-generated trades to prevent double processing
            if trade_id in self.trading_engine.bot_trade_ids:
                logger.debug(f"🤖 Skipping bot-generated trade: {trade_id}")
                return
            
            # Record trade in stats and get action type using enhanced tracking
            stats = self.trading_engine.get_stats()
            if stats:
                action_type = stats.record_trade_with_enhanced_tracking(symbol, side, amount, price, trade_id, "external")
                
                # Send enhanced notification based on action type
                await self._send_enhanced_trade_notification(symbol, side, amount, price, action_type, timestamp)
                
                logger.info(f"📋 Processed external trade: {side} {amount} {symbol} @ ${price} ({action_type})")
            
        except Exception as e:
            logger.error(f"❌ Error processing external trade: {e}")
    
    async def _send_enhanced_trade_notification(self, symbol: str, side: str, amount: float, 
                                                price: float, action_type: str, timestamp: str):
        """Send enhanced notification for external trades."""
        try:
            # Send through notification manager if available
            if self.notification_manager:
                await self.notification_manager.send_external_trade_notification(
                    symbol, side, amount, price, action_type, timestamp
                )
            else:
                # Fallback to logging if notification manager not available
                logger.info(f"📢 External trade notification: {action_type} for {symbol.split('/')[0]}")
            
        except Exception as e:
            logger.error(f"❌ Error sending external trade notification: {e}") 