#!/usr/bin/env python3
"""
Trading Engine - Handles order execution, position tracking, and trading logic.
"""

import os
import json
import logging
from typing import Dict, Any, Optional, Tuple, List
from datetime import datetime

from src.config.config import Config
from src.clients.hyperliquid_client import HyperliquidClient
from src.trading.trading_stats import TradingStats

logger = logging.getLogger(__name__)

class TradingEngine:
    """Handles all trading operations, order execution, and position tracking."""
    
    def __init__(self):
        """Initialize the trading engine."""
        self.client = HyperliquidClient()
        self.stats = None
        
        # State persistence
        self.state_file = "trading_engine_state.json"
        
        # Position and order tracking
        self.bot_trade_ids = set()  # Track bot-generated trades
        self.pending_stop_losses = {}  # Pending stop loss orders
        
        # Load state and initialize stats
        self._load_state()
        self._initialize_stats()
        
    def _load_state(self):
        """Load trading engine state from disk."""
        try:
            if os.path.exists(self.state_file):
                with open(self.state_file, 'r') as f:
                    state_data = json.load(f)
                
                self.bot_trade_ids = set(state_data.get('bot_trade_ids', []))
                self.pending_stop_losses = state_data.get('pending_stop_losses', {})
                
                logger.info(f"🔄 Loaded trading engine state: {len(self.bot_trade_ids)} tracked trades")
        except Exception as e:
            logger.error(f"Error loading trading engine state: {e}")
            self.bot_trade_ids = set()
            self.pending_stop_losses = {}
    
    def _save_state(self):
        """Save trading engine state to disk."""
        try:
            state_data = {
                'bot_trade_ids': list(self.bot_trade_ids),
                'pending_stop_losses': self.pending_stop_losses,
                'last_updated': datetime.now().isoformat()
            }
            
            with open(self.state_file, 'w') as f:
                json.dump(state_data, f, indent=2, default=str)
        except Exception as e:
            logger.error(f"Error saving trading engine state: {e}")
    
    def _initialize_stats(self):
        """Initialize trading statistics."""
        try:
            self.stats = TradingStats()
            
            # Set initial balance
            balance = self.client.get_balance()
            if balance and balance.get('total'):
                usdc_balance = float(balance['total'].get('USDC', 0))
                self.stats.set_initial_balance(usdc_balance)
        except Exception as e:
            logger.error(f"Could not initialize trading stats: {e}")
    
    def get_balance(self) -> Optional[Dict[str, Any]]:
        """Get account balance."""
        return self.client.get_balance()
    
    def get_positions(self) -> Optional[List[Dict[str, Any]]]:
        """Get current positions."""
        return self.client.get_positions()
    
    def get_orders(self) -> Optional[List[Dict[str, Any]]]:
        """Get open orders."""
        return self.client.get_open_orders()
    
    def get_market_data(self, symbol: str) -> Optional[Dict[str, Any]]:
        """Get market data for a symbol."""
        return self.client.get_market_data(symbol)
    
    def find_position(self, token: str) -> Optional[Dict[str, Any]]:
        """Find an open position for a token."""
        symbol = f"{token}/USDC:USDC"
        positions = self.get_positions()
        
        if positions:
            for position in positions:
                if (position.get('symbol') == symbol and 
                    float(position.get('contracts', 0)) != 0):
                    return position
        return None
    
    def get_position_direction(self, position: Dict[str, Any]) -> Tuple[str, str, float]:
        """
        Get position direction info from CCXT position data.
        Returns: (position_type, exit_side, contracts_abs)
        """
        contracts = float(position.get('contracts', 0))
        side_field = position.get('side', '').lower()
        
        if side_field == 'long':
            return "LONG", "sell", abs(contracts)
        elif side_field == 'short':
            return "SHORT", "buy", abs(contracts)
        else:
            # Fallback to contracts sign (less reliable)
            if contracts > 0:
                return "LONG", "sell", abs(contracts)
            else:
                return "SHORT", "buy", abs(contracts)
    
    async def execute_long_order(self, token: str, usdc_amount: float, 
                                price: Optional[float] = None, 
                                stop_loss_price: Optional[float] = None) -> Dict[str, Any]:
        """Execute a long order."""
        symbol = f"{token}/USDC:USDC"
        
        try:
            # Validate inputs
            if usdc_amount <= 0:
                return {"success": False, "error": "Invalid USDC amount"}
            
            # Get market data for price validation
            market_data = self.get_market_data(symbol)
            if not market_data:
                return {"success": False, "error": f"Could not fetch market data for {token}"}
            
            current_price = float(market_data['ticker'].get('last', 0))
            if current_price <= 0:
                return {"success": False, "error": f"Invalid current price for {token}"}
            
            # Calculate token amount
            if price:
                # Limit order - use specified price
                token_amount = usdc_amount / price
            else:
                # Market order - use current price
                token_amount = usdc_amount / current_price
                price = current_price
            
            # Execute the order
            if price == current_price:
                # Market order
                order = self.client.place_market_order(symbol, 'buy', token_amount)
            else:
                # Limit order
                order = self.client.place_limit_order(symbol, 'buy', token_amount, price)
            
            if order:
                # Track the order
                order_id = order.get('id', 'N/A')
                actual_price = order.get('average', price)
                
                if order_id != 'N/A':
                    self.bot_trade_ids.add(order_id)
                
                # Record in stats
                action_type = self.stats.record_trade_with_enhanced_tracking(
                    symbol, 'buy', token_amount, actual_price, order_id, "bot"
                )
                
                # Handle stop loss if specified
                if stop_loss_price and order_id != 'N/A':
                    self.pending_stop_losses[order_id] = {
                        'token': token,
                        'stop_price': stop_loss_price,
                        'side': 'sell',  # Exit side for long position
                        'amount': token_amount,
                        'order_type': 'stop_loss'
                    }
                
                self._save_state()
                
                return {
                    "success": True,
                    "order": order,
                    "action_type": action_type,
                    "token_amount": token_amount,
                    "actual_price": actual_price,
                    "stop_loss_pending": stop_loss_price is not None
                }
            else:
                return {"success": False, "error": "Order execution failed"}
                
        except Exception as e:
            logger.error(f"Error executing long order: {e}")
            return {"success": False, "error": str(e)}
    
    async def execute_short_order(self, token: str, usdc_amount: float, 
                                 price: Optional[float] = None, 
                                 stop_loss_price: Optional[float] = None) -> Dict[str, Any]:
        """Execute a short order."""
        symbol = f"{token}/USDC:USDC"
        
        try:
            # Similar to long order but with sell side
            if usdc_amount <= 0:
                return {"success": False, "error": "Invalid USDC amount"}
            
            market_data = self.get_market_data(symbol)
            if not market_data:
                return {"success": False, "error": f"Could not fetch market data for {token}"}
            
            current_price = float(market_data['ticker'].get('last', 0))
            if current_price <= 0:
                return {"success": False, "error": f"Invalid current price for {token}"}
            
            # Calculate token amount
            if price:
                token_amount = usdc_amount / price
            else:
                token_amount = usdc_amount / current_price
                price = current_price
            
            # Execute the order
            if price == current_price:
                order = self.client.place_market_order(symbol, 'sell', token_amount)
            else:
                order = self.client.place_limit_order(symbol, 'sell', token_amount, price)
            
            if order:
                order_id = order.get('id', 'N/A')
                actual_price = order.get('average', price)
                
                if order_id != 'N/A':
                    self.bot_trade_ids.add(order_id)
                
                action_type = self.stats.record_trade_with_enhanced_tracking(
                    symbol, 'sell', token_amount, actual_price, order_id, "bot"
                )
                
                # Handle stop loss if specified
                if stop_loss_price and order_id != 'N/A':
                    self.pending_stop_losses[order_id] = {
                        'token': token,
                        'stop_price': stop_loss_price,
                        'side': 'buy',  # Exit side for short position
                        'amount': token_amount,
                        'order_type': 'stop_loss'
                    }
                
                self._save_state()
                
                return {
                    "success": True,
                    "order": order,
                    "action_type": action_type,
                    "token_amount": token_amount,
                    "actual_price": actual_price,
                    "stop_loss_pending": stop_loss_price is not None
                }
            else:
                return {"success": False, "error": "Order execution failed"}
                
        except Exception as e:
            logger.error(f"Error executing short order: {e}")
            return {"success": False, "error": str(e)}
    
    async def execute_exit_order(self, token: str) -> Dict[str, Any]:
        """Execute an exit order to close a position."""
        position = self.find_position(token)
        if not position:
            return {"success": False, "error": f"No open position found for {token}"}
        
        try:
            symbol = f"{token}/USDC:USDC"
            position_type, exit_side, contracts = self.get_position_direction(position)
            
            # Execute market order to close position
            order = self.client.place_market_order(symbol, exit_side, contracts)
            
            if order:
                order_id = order.get('id', 'N/A')
                actual_price = order.get('average', 0)
                
                if order_id != 'N/A':
                    self.bot_trade_ids.add(order_id)
                
                action_type = self.stats.record_trade_with_enhanced_tracking(
                    symbol, exit_side, contracts, actual_price, order_id, "bot"
                )
                
                self._save_state()
                
                return {
                    "success": True,
                    "order": order,
                    "action_type": action_type,
                    "position_type": position_type,
                    "contracts": contracts,
                    "actual_price": actual_price
                }
            else:
                return {"success": False, "error": "Exit order execution failed"}
                
        except Exception as e:
            logger.error(f"Error executing exit order: {e}")
            return {"success": False, "error": str(e)}
    
    async def execute_stop_loss_order(self, token: str, stop_price: float) -> Dict[str, Any]:
        """Execute a stop loss order."""
        position = self.find_position(token)
        if not position:
            return {"success": False, "error": f"No open position found for {token}"}
        
        try:
            symbol = f"{token}/USDC:USDC"
            position_type, exit_side, contracts = self.get_position_direction(position)
            entry_price = float(position.get('entryPrice', 0))
            
            # Validate stop loss price based on position direction
            if position_type == "LONG" and stop_price >= entry_price:
                return {"success": False, "error": "Stop loss price should be below entry price for long positions"}
            elif position_type == "SHORT" and stop_price <= entry_price:
                return {"success": False, "error": "Stop loss price should be above entry price for short positions"}
            
            # Place limit order at stop loss price
            order = self.client.place_limit_order(symbol, exit_side, contracts, stop_price)
            
            if order:
                order_id = order.get('id', 'N/A')
                
                if order_id != 'N/A':
                    self.bot_trade_ids.add(order_id)
                
                self._save_state()
                
                return {
                    "success": True,
                    "order": order,
                    "position_type": position_type,
                    "contracts": contracts,
                    "stop_price": stop_price
                }
            else:
                return {"success": False, "error": "Stop loss order placement failed"}
                
        except Exception as e:
            logger.error(f"Error executing stop loss order: {e}")
            return {"success": False, "error": str(e)}
    
    async def execute_take_profit_order(self, token: str, profit_price: float) -> Dict[str, Any]:
        """Execute a take profit order."""
        position = self.find_position(token)
        if not position:
            return {"success": False, "error": f"No open position found for {token}"}
        
        try:
            symbol = f"{token}/USDC:USDC"
            position_type, exit_side, contracts = self.get_position_direction(position)
            entry_price = float(position.get('entryPrice', 0))
            
            # Validate take profit price based on position direction
            if position_type == "LONG" and profit_price <= entry_price:
                return {"success": False, "error": "Take profit price should be above entry price for long positions"}
            elif position_type == "SHORT" and profit_price >= entry_price:
                return {"success": False, "error": "Take profit price should be below entry price for short positions"}
            
            # Place limit order at take profit price
            order = self.client.place_limit_order(symbol, exit_side, contracts, profit_price)
            
            if order:
                order_id = order.get('id', 'N/A')
                
                if order_id != 'N/A':
                    self.bot_trade_ids.add(order_id)
                
                self._save_state()
                
                return {
                    "success": True,
                    "order": order,
                    "position_type": position_type,
                    "contracts": contracts,
                    "profit_price": profit_price
                }
            else:
                return {"success": False, "error": "Take profit order placement failed"}
                
        except Exception as e:
            logger.error(f"Error executing take profit order: {e}")
            return {"success": False, "error": str(e)}
    
    async def cancel_all_orders(self, token: str) -> Dict[str, Any]:
        """Cancel all orders for a token."""
        try:
            symbol = f"{token}/USDC:USDC"
            result = self.client.cancel_all_orders(symbol)
            
            return {
                "success": True,
                "result": result
            }
        except Exception as e:
            logger.error(f"Error cancelling orders: {e}")
            return {"success": False, "error": str(e)}
    
    # Alias methods for consistency with command handlers
    async def execute_sl_order(self, token: str, stop_price: float) -> Dict[str, Any]:
        """Alias for execute_stop_loss_order."""
        return await self.execute_stop_loss_order(token, stop_price)
    
    async def execute_tp_order(self, token: str, profit_price: float) -> Dict[str, Any]:
        """Alias for execute_take_profit_order."""
        return await self.execute_take_profit_order(token, profit_price)
    
    async def execute_coo_order(self, token: str) -> Dict[str, Any]:
        """Alias for cancel_all_orders."""
        return await self.cancel_all_orders(token)
    
    def is_bot_trade(self, trade_id: str) -> bool:
        """Check if a trade was generated by this bot."""
        return trade_id in self.bot_trade_ids
    
    def get_stats(self) -> TradingStats:
        """Get trading statistics object."""
        return self.stats 