#!/usr/bin/env python3
"""
Trading Statistics Tracker

Tracks and calculates comprehensive trading statistics including:
- Balance tracking
- Trade history
- P&L analysis
- Win/Loss ratios
- Risk metrics (Sharpe, Sortino)
- Performance metrics
"""

import json
import os
import logging
from datetime import datetime, timedelta, timezone
from typing import Dict, List, Any, Optional, Tuple
import numpy as np
import math
from collections import defaultdict

logger = logging.getLogger(__name__)

class TradingStats:
    """Comprehensive trading statistics tracker."""
    
    def __init__(self, stats_file: str = "data/trading_stats.json"):
        """Initialize the stats tracker."""
        self.stats_file = stats_file
        
        # Ensure the data directory exists
        data_dir = os.path.dirname(self.stats_file)
        if data_dir and not os.path.exists(data_dir):
            os.makedirs(data_dir)
            logger.info(f"Created data directory: {data_dir}")
            
        self.data = self._load_stats()
        self._initialize_if_empty()
    
    def _load_stats(self) -> Dict[str, Any]:
        """Load stats from file."""
        try:
            if os.path.exists(self.stats_file):
                with open(self.stats_file, 'r') as f:
                    loaded_data = json.load(f)
                    # Basic validation
                    if isinstance(loaded_data, dict) and 'trades' in loaded_data:
                        # Ensure necessary nested structures exist
                        loaded_data.setdefault('positions', {})
                        loaded_data.setdefault('daily_balances', [])
                        loaded_data.setdefault('balance_adjustments', [])
                        return loaded_data
                    else:
                        logger.warning(f"Stats file {self.stats_file} has invalid format. Re-initializing.")
                        return {}
        except json.JSONDecodeError as e:
            logger.error(f"Error decoding JSON from {self.stats_file}: {e}. Re-initializing.")
            return {}
        except Exception as e:
            logger.error(f"Error loading stats from {self.stats_file}: {e}")
        return {}
    
    def _save_stats(self):
        """Save stats to file."""
        try:
            self.data['last_updated'] = datetime.now(timezone.utc).isoformat()
            with open(self.stats_file, 'w') as f:
                json.dump(self.data, f, indent=2)
        except Exception as e:
            logger.error(f"Error saving stats to {self.stats_file}: {e}")
    
    def _initialize_if_empty(self):
        """Initialize the data structure if it's empty or missing keys."""
        if not self.data or not all(k in self.data for k in ['trades', 'initial_balance', 'start_date']):
            self.data = {
                'trades': [],
                'initial_balance': 0.0,
                'start_date': datetime.now(timezone.utc).isoformat(),
                'positions': {},  # For enhanced position tracking
                'daily_balances': [], # Store {date, balance} for P&L over time
                'balance_adjustments': [] # Store {type, amount, timestamp, id}
            }
            self._save_stats()
            logger.info("Initialized new trading stats file.")
        else:
            # Ensure all necessary keys are present for older stats files
            self.data.setdefault('positions', {})
            self.data.setdefault('daily_balances', [])
            self.data.setdefault('balance_adjustments', [])
            logger.info(f"Loaded trading stats. Last update: {self.data.get('last_updated', 'N/A')}")
    
    def set_initial_balance(self, balance: float):
        """Set the initial balance when bot starts."""
        if self.data.get('initial_balance', 0) == 0:
            self.data['initial_balance'] = balance
            self.data['start_date'] = datetime.now(timezone.utc).isoformat()
            logger.info(f"Initial balance set to: ${balance:.2f}")
            self._save_stats()
    
    def record_balance(self, balance: float):
        """Record daily balance snapshot."""
        today = datetime.now().date().isoformat()
        
        # Check if we already have today's balance
        for entry in self.data['daily_balances']:
            if entry['date'] == today:
                entry['balance'] = balance
                entry['timestamp'] = datetime.now().isoformat()
                self._save_stats()
                return
        
        # Add new daily balance
        self.data['daily_balances'].append({
            'date': today,
            'balance': balance,
            'timestamp': datetime.now().isoformat()
        })
        self._save_stats()
    
    def record_trade(self, symbol: str, side: str, amount: float, price: float, 
                    order_id: str = None, trade_type: str = "manual"):
        """Record a trade."""
        trade = {
            'timestamp': datetime.now().isoformat(),
            'symbol': symbol,
            'side': side.lower(),
            'amount': amount,
            'price': price,
            'value': amount * price,
            'order_id': order_id,
            'type': trade_type,
            'pnl': 0.0  # Will be calculated when position is closed
        }
        
        self.data['trades'].append(trade)
        self.data['last_update'] = datetime.now().isoformat()
        
        # Update period stats if this is a sell order (position close)
        if side.lower() == 'sell':
            self._update_period_stats()
        
        self._save_stats()
        
        logger.info(f"Recorded trade: {side} {amount} {symbol} @ ${price:.2f}")
    
    def get_enhanced_position_state(self, symbol: str) -> Dict[str, Any]:
        """Get current enhanced position state for a symbol."""
        # Initialize enhanced_positions if not present (for backward compatibility)
        if 'positions' not in self.data:
            self.data['positions'] = {}
        
        if symbol not in self.data['positions']:
            self.data['positions'][symbol] = {
                'contracts': 0.0,
                'avg_entry_price': 0.0,
                'total_cost_basis': 0.0,
                'total_sell_value': 0.0,
                'entry_count': 0,
                'last_entry_timestamp': None,
                'realized_pnl': 0.0,
                'side': None
            }
        return self.data['positions'][symbol]
    
    def update_enhanced_position_state(self, symbol: str, side: str, amount: float, price: float, timestamp: str = None):
        """Update enhanced position state with a new trade and return action type."""
        if timestamp is None:
            timestamp = datetime.now().isoformat()
            
        position = self.get_enhanced_position_state(symbol)
        
        if side.lower() == 'buy':
            # Adding to long position or reducing short position
            if position['contracts'] >= 0:
                # Opening/adding to long position
                old_contracts = position['contracts']
                
                position['contracts'] += amount
                position['total_cost_basis'] += amount * price
                position['avg_entry_price'] = position['total_cost_basis'] / position['contracts'] if position['contracts'] > 0 else 0
                position['entry_count'] += 1
                position['last_entry_timestamp'] = timestamp
                
                logger.info(f"📈 Enhanced position updated: {symbol} LONG {position['contracts']:.6f} @ avg ${position['avg_entry_price']:.2f}")
                action_type = 'long_opened' if old_contracts == 0 else 'long_increased'
            else:
                # Reducing short position
                reduction = min(amount, abs(position['contracts']))
                position['contracts'] += reduction
                
                if position['contracts'] >= 0:
                    # Short position fully closed or flipped to long
                    if position['contracts'] == 0:
                        self._reset_enhanced_position_state(symbol)
                        action_type = 'short_closed'
                    else:
                        # Flipped to long - need to track new long position
                        remaining_amount = amount - reduction
                        position['contracts'] = remaining_amount
                        position['total_cost_basis'] = remaining_amount * price
                        position['avg_entry_price'] = price
                        position['entry_count'] = 1
                        position['last_entry_timestamp'] = timestamp
                        action_type = 'short_closed_and_long_opened'
                else:
                    action_type = 'short_reduced'
        
        elif side.lower() == 'sell':
            # Adding to short position or reducing long position
            if position['contracts'] <= 0:
                # Opening/adding to short position
                old_contracts = position['contracts']
                position['contracts'] -= amount
                position['entry_count'] += 1
                position['last_entry_timestamp'] = timestamp
                
                logger.info(f"📉 Enhanced position updated: {symbol} SHORT {abs(position['contracts']):.6f} @ ${price:.2f}")
                action_type = 'short_opened' if old_contracts == 0 else 'short_increased'
            else:
                # Reducing long position
                reduction = min(amount, position['contracts'])
                position['contracts'] -= reduction
                
                # Adjust cost basis proportionally
                if position['contracts'] > 0:
                    reduction_ratio = reduction / (position['contracts'] + reduction)
                    position['total_cost_basis'] *= (1 - reduction_ratio)
                    action_type = 'long_reduced'
                else:
                    # Long position fully closed
                    if position['contracts'] == 0:
                        self._reset_enhanced_position_state(symbol)
                        action_type = 'long_closed'
                    else:
                        # Flipped to short
                        remaining_amount = amount - reduction
                        position['contracts'] = -remaining_amount
                        position['entry_count'] = 1
                        position['last_entry_timestamp'] = timestamp
                        action_type = 'long_closed_and_short_opened'
        
        position['last_entry_timestamp'] = timestamp
        self._save_stats()
        return action_type
    
    def _reset_enhanced_position_state(self, symbol: str):
        """Reset enhanced position state when position is fully closed."""
        if 'positions' in self.data and symbol in self.data['positions']:
            del self.data['positions'][symbol]
    
    def calculate_enhanced_position_pnl(self, symbol: str, exit_amount: float, exit_price: float) -> Dict[str, float]:
        """Calculate P&L for a position exit using enhanced tracking."""
        position = self.get_enhanced_position_state(symbol)
        
        if position['contracts'] == 0:
            return {'pnl': 0.0, 'pnl_percent': 0.0, 'avg_entry_price': 0.0, 'closed_contracts': 0.0}
        
        avg_entry = position['avg_entry_price']
        
        if position['contracts'] > 0:  # Long position
            pnl = exit_amount * (exit_price - avg_entry)
        else:  # Short position
            pnl = exit_amount * (avg_entry - exit_price)
        
        cost_basis = exit_amount * avg_entry
        pnl_percent = (pnl / cost_basis * 100) if cost_basis > 0 else 0
        
        return {
            'pnl': pnl,
            'pnl_percent': pnl_percent,
            'avg_entry_price': avg_entry,
            'closed_contracts': exit_amount
        }
    
    def record_trade_with_enhanced_tracking(self, symbol: str, side: str, amount: float, price: float, 
                                          order_id: str = None, trade_type: str = "manual"):
        """Record a trade with enhanced position tracking and return action type."""
        # Record the trade normally
        self.record_trade(symbol, side, amount, price, order_id, trade_type)
        
        # Update enhanced position state and return action type
        return self.update_enhanced_position_state(symbol, side, amount, price)
    
    def calculate_trade_pnl(self) -> List[Dict[str, Any]]:
        """Calculate P&L for completed trades using FIFO method."""
        trades_with_pnl = []
        positions = {}  # Track open positions by symbol
        
        for trade in self.data['trades']:
            symbol = trade['symbol']
            side = trade['side']
            amount = trade['amount']
            price = trade['price']
            
            if symbol not in positions:
                positions[symbol] = {'amount': 0, 'avg_price': 0, 'total_cost': 0}
            
            pos = positions[symbol]
            
            if side == 'buy':
                # Add to position
                pos['total_cost'] += amount * price
                pos['amount'] += amount
                pos['avg_price'] = pos['total_cost'] / pos['amount'] if pos['amount'] > 0 else 0
                
                trade_copy = trade.copy()
                trade_copy['pnl'] = 0  # No PnL on opening
                trades_with_pnl.append(trade_copy)
                
            elif side == 'sell':
                # Reduce position and calculate PnL
                if pos['amount'] > 0:
                    sold_amount = min(amount, pos['amount'])
                    pnl = sold_amount * (price - pos['avg_price'])
                    
                    # Update position
                    pos['amount'] -= sold_amount
                    if pos['amount'] > 0:
                        pos['total_cost'] = pos['amount'] * pos['avg_price']
                    else:
                        pos['total_cost'] = 0
                        pos['avg_price'] = 0
                    
                    trade_copy = trade.copy()
                    trade_copy['pnl'] = pnl
                    trade_copy['sold_amount'] = sold_amount
                    trades_with_pnl.append(trade_copy)
                else:
                    # Short selling (negative PnL calculation)
                    trade_copy = trade.copy()
                    trade_copy['pnl'] = 0  # Would need more complex logic for shorts
                    trades_with_pnl.append(trade_copy)
        
        return trades_with_pnl
    
    def get_basic_stats(self, current_balance: Optional[float] = None) -> Dict[str, Any]:
        """Get basic trading statistics."""
        if not self.data['trades']:
            return {
                'total_trades': 0,
                'completed_trades': 0,
                'buy_trades': 0,
                'sell_trades': 0,
                'initial_balance': self.data.get('initial_balance', 0),
                'current_balance': 0,
                'total_pnl': 0,
                'days_active': 0,
                'start_date': datetime.now(timezone.utc).strftime('%Y-%m-%d') if 'start_date' not in self.data else datetime.fromisoformat(self.data['start_date']).strftime('%Y-%m-%d'),
                'last_trade': None
            }
        
        completed_cycles = self.calculate_completed_trade_cycles()
        total_pnl = sum(cycle['total_pnl'] for cycle in completed_cycles)
        
        # Calculate days active
        start_date = datetime.fromisoformat(self.data['start_date'])
        days_active = (datetime.now(timezone.utc) - start_date).days + 1
        
        return {
            'total_trades': len(self.data['trades']),  # Total individual orders placed
            'completed_trades': len(completed_cycles),  # Actual completed trade cycles
            'buy_trades': len([t for t in self.data['trades'] if t['side'] == 'buy']),
            'sell_trades': len([t for t in self.data['trades'] if t['side'] == 'sell']),
            'initial_balance': self.data.get('initial_balance', 0),
            'total_pnl': total_pnl,
            'days_active': days_active,
            'start_date': start_date.strftime('%Y-%m-%d'),
            'last_trade': self.data['trades'][-1]['timestamp'] if self.data['trades'] else None
        }
    
    def get_performance_stats(self) -> Dict[str, Any]:
        """Calculate advanced performance statistics."""
        completed_cycles = self.calculate_completed_trade_cycles()
        
        if not completed_cycles:
            return {
                'win_rate': 0,
                'profit_factor': 0,
                'avg_win': 0,
                'avg_loss': 0,
                'largest_win': 0,
                'largest_loss': 0,
                'consecutive_wins': 0,
                'consecutive_losses': 0,
                'total_wins': 0,
                'total_losses': 0,
                'expectancy': 0
            }
        
        # Separate wins and losses by trade cycles
        wins = [c['total_pnl'] for c in completed_cycles if c['total_pnl'] > 0]
        losses = [abs(c['total_pnl']) for c in completed_cycles if c['total_pnl'] < 0]
        
        # Basic metrics
        total_wins = len(wins)
        total_losses = len(losses)
        total_completed = total_wins + total_losses
        
        win_rate = (total_wins / total_completed * 100) if total_completed > 0 else 0
        
        # Profit metrics
        total_profit = sum(wins) if wins else 0
        total_loss = sum(losses) if losses else 0
        profit_factor = (total_profit / total_loss) if total_loss > 0 else float('inf') if total_profit > 0 else 0
        
        avg_win = np.mean(wins) if wins else 0
        avg_loss = np.mean(losses) if losses else 0
        
        largest_win = max(wins) if wins else 0
        largest_loss = max(losses) if losses else 0
        
        # Consecutive wins/losses (by trade cycles)
        consecutive_wins = 0
        consecutive_losses = 0
        current_wins = 0
        current_losses = 0
        
        for cycle in completed_cycles:
            if cycle['total_pnl'] > 0:
                current_wins += 1
                current_losses = 0
                consecutive_wins = max(consecutive_wins, current_wins)
            else:
                current_losses += 1
                current_wins = 0
                consecutive_losses = max(consecutive_losses, current_losses)
        
        return {
            'win_rate': win_rate,
            'profit_factor': profit_factor,
            'avg_win': avg_win,
            'avg_loss': avg_loss,
            'largest_win': largest_win,
            'largest_loss': largest_loss,
            'consecutive_wins': consecutive_wins,
            'consecutive_losses': consecutive_losses,
            'total_wins': total_wins,
            'total_losses': total_losses,
            'expectancy': avg_win * (win_rate/100) - avg_loss * ((100-win_rate)/100)
        }
    
    def get_risk_metrics(self) -> Dict[str, Any]:
        """Calculate risk-adjusted metrics."""
        if not self.data['daily_balances'] or len(self.data['daily_balances']) < 2:
            return {
                'sharpe_ratio': 0,
                'sortino_ratio': 0,
                'max_drawdown': 0,
                'volatility': 0,
                'var_95': 0
            }
        
        # Calculate daily returns
        balances = [entry['balance'] for entry in self.data['daily_balances']]
        returns = []
        for i in range(1, len(balances)):
            daily_return = (balances[i] - balances[i-1]) / balances[i-1] if balances[i-1] > 0 else 0
            returns.append(daily_return)
        
        if not returns:
            return {
                'sharpe_ratio': 0,
                'sortino_ratio': 0,
                'max_drawdown': 0,
                'volatility': 0,
                'var_95': 0
            }
        
        returns = np.array(returns)
        
        # Risk-free rate (assume 2% annually, convert to daily)
        risk_free_rate = 0.02 / 365
        
        # Sharpe Ratio
        excess_returns = returns - risk_free_rate
        sharpe_ratio = np.mean(excess_returns) / np.std(returns) * np.sqrt(365) if np.std(returns) > 0 else 0
        
        # Sortino Ratio (only downside volatility)
        downside_returns = returns[returns < 0]
        downside_std = np.std(downside_returns) if len(downside_returns) > 0 else 0
        sortino_ratio = np.mean(excess_returns) / downside_std * np.sqrt(365) if downside_std > 0 else 0
        
        # Maximum Drawdown
        cumulative = np.cumprod(1 + returns)
        running_max = np.maximum.accumulate(cumulative)
        drawdown = (cumulative - running_max) / running_max
        max_drawdown = np.min(drawdown) * 100
        
        # Volatility (annualized)
        volatility = np.std(returns) * np.sqrt(365) * 100
        
        # Value at Risk (95%)
        var_95 = np.percentile(returns, 5) * 100
        
        return {
            'sharpe_ratio': sharpe_ratio,
            'sortino_ratio': sortino_ratio,
            'max_drawdown': abs(max_drawdown),
            'volatility': volatility,
            'var_95': abs(var_95)
        }
    
    def get_comprehensive_stats(self, current_balance: float = None) -> Dict[str, Any]:
        """Get all statistics combined."""
        if current_balance:
            self.record_balance(current_balance)
        
        basic = self.get_basic_stats(current_balance)
        performance = self.get_performance_stats()
        risk = self.get_risk_metrics()
        
        # Calculate total return
        initial_balance = basic['initial_balance']
        total_return = 0
        if current_balance and initial_balance > 0:
            total_return = ((current_balance - initial_balance) / initial_balance) * 100
        
        return {
            'basic': basic,
            'performance': performance,
            'risk': risk,
            'current_balance': current_balance or 0,
            'total_return': total_return,
            'last_updated': datetime.now(timezone.utc).isoformat()
        }
    
    def format_stats_message(self, current_balance: float = None) -> str:
        """Format stats for Telegram display."""
        try:
            stats = self.get_comprehensive_stats(current_balance)
            
            basic = stats['basic']
            perf = stats['performance']
            risk = stats['risk']
            
            # Check if we have completed trades or just open positions
            completed_trades = basic.get('completed_trades', 0)
            total_orders = basic.get('total_trades', 0)
            
            # Calculate key metrics
            total_pnl = current_balance - basic['initial_balance'] if basic['initial_balance'] > 0 else 0
            total_return = (total_pnl / basic['initial_balance'] * 100) if basic['initial_balance'] > 0 else 0
            win_rate = (perf['win_rate'] / 100) if completed_trades > 0 else 0
            
            # Calculate average trade metrics
            avg_win = perf['avg_win']
            avg_loss = perf['avg_loss']
            profit_factor = perf['profit_factor']
            
            # Get balance adjustments summary
            adjustments_summary = self.get_balance_adjustments_summary()
            
            # P&L emoji
            pnl_emoji = "🟢" if total_pnl >= 0 else "🔴"
            
            stats_text = f"""
📊 <b>Trading Statistics</b>

💰 <b>Account Overview:</b>
• Current Balance: ${current_balance:,.2f}
• Initial Balance: ${basic['initial_balance']:,.2f}
• {pnl_emoji} Total P&L: ${total_pnl:,.2f} ({total_return:+.2f}%)

📈 <b>Trading Activity:</b>
• Total Orders: {total_orders}
• Completed Trades: {completed_trades}
• Open Positions: {len([p for p in self.data.get('positions', {}).values() if p.get('contracts', 0) != 0])}
• Days Active: {(datetime.now(timezone.utc) - datetime.fromisoformat(basic['start_date'])).days + 1}

🏆 <b>Performance Metrics:</b>
• Win Rate: {win_rate:.1f}% ({perf['total_wins']}W/{perf['total_losses']}L)
• Profit Factor: {profit_factor:.2f}
• Avg Win: ${avg_win:.2f} | Avg Loss: ${avg_loss:.2f}
• Largest Win: ${perf['largest_win']:.2f} | Largest Loss: ${perf['largest_loss']:.2f}
"""
            
            # Add balance adjustments section if there are any
            if adjustments_summary['adjustment_count'] > 0:
                adj_emoji = "💰" if adjustments_summary['net_adjustment'] >= 0 else "💸"
                stats_text += f"""
💰 <b>Balance Adjustments:</b>
• Deposits: ${adjustments_summary['total_deposits']:,.2f}
• Withdrawals: ${adjustments_summary['total_withdrawals']:,.2f}
• {adj_emoji} Net: ${adjustments_summary['net_adjustment']:,.2f} ({adjustments_summary['adjustment_count']} transactions)

"""
            
            # Calculate trade volume safely
            stats_text += f"""
🎯 <b>Trade Distribution:</b>
• Buy Orders: {basic['buy_trades']} | Sell Orders: {basic['sell_trades']}
• Volume Traded: ${sum(t['value'] for t in self.data['trades'] if t['side'] == 'sell'):,.2f}
• Avg Trade Size: ${(sum(t['value'] for t in self.data['trades'] if t['side'] == 'sell') / len([t for t in self.data['trades'] if t['side'] == 'sell'])):.2f}

⏰ <b>Session Info:</b>
• Started: {basic['start_date']}
• Last Update: {datetime.now(timezone.utc).strftime('%H:%M:%S')}
            """
            
            return stats_text.strip()
                
        except Exception as e:
            # Fallback message if stats calculation fails
            logger.error(f"Error formatting stats message: {e}")
            return f"""
📊 <b>Trading Statistics</b>

❌ <b>Error loading statistics</b>

Please try again in a moment. If the issue persists, contact support.

🔧 <b>Debug info:</b> {str(e)[:100]}
            """.strip()
    
    def get_recent_trades(self, limit: int = 10) -> List[Dict[str, Any]]:
        """Get recent trades."""
        return self.data['trades'][-limit:] if self.data['trades'] else []

    def get_token_performance(self) -> Dict[str, Dict[str, Any]]:
        """Get performance statistics grouped by token."""
        completed_cycles = self.calculate_completed_trade_cycles()
        
        # Group cycles by token
        token_cycles = {}
        
        for cycle in completed_cycles:
            token = cycle['token']
            
            if token not in token_cycles:
                token_cycles[token] = []
            token_cycles[token].append(cycle)
        
        # Calculate performance for each token
        token_performance = {}
        
        for token, cycles in token_cycles.items():
            # Only include tokens that have actual completed trade cycles
            if not cycles:
                continue  # Skip tokens with no completed cycles
            
            # Calculate metrics for completed cycles
            total_pnl = sum(c['total_pnl'] for c in cycles)
            total_volume = sum(c['sell_value'] for c in cycles)  # Use sell value for volume
            pnl_percentage = (total_pnl / total_volume * 100) if total_volume > 0 else 0.0
            
            # Win/loss analysis
            wins = [c['total_pnl'] for c in cycles if c['total_pnl'] > 0]
            losses = [abs(c['total_pnl']) for c in cycles if c['total_pnl'] < 0]
            
            total_wins = len(wins)
            total_losses = len(losses)
            total_completed = total_wins + total_losses
            
            win_rate = (total_wins / total_completed * 100) if total_completed > 0 else 0.0
            
            # Calculate profit factor
            total_profit = sum(wins) if wins else 0
            total_loss = sum(losses) if losses else 0
            profit_factor = (total_profit / total_loss) if total_loss > 0 else float('inf') if total_profit > 0 else 0
            
            # Calculate expectancy
            avg_win = total_profit / total_wins if total_wins > 0 else 0
            avg_loss = total_loss / total_losses if total_losses > 0 else 0
            expectancy = avg_win * (win_rate / 100) - avg_loss * ((100 - win_rate) / 100)
            
            # Largest win/loss
            largest_win = max(wins) if wins else 0
            largest_loss = max(losses) if losses else 0
            
            token_performance[token] = {
                'total_pnl': total_pnl,
                'pnl_percentage': pnl_percentage,
                'completed_trades': len(cycles),  # Now correctly counts trade cycles
                'total_volume': total_volume,
                'win_rate': win_rate,
                'total_wins': total_wins,
                'total_losses': total_losses,
                'profit_factor': profit_factor,
                'expectancy': expectancy,
                'largest_win': largest_win,
                'largest_loss': largest_loss,
                'avg_win': avg_win,
                'avg_loss': avg_loss,
                'cycles': cycles  # Include cycle details for further analysis
            }
        
        return token_performance

    def get_token_detailed_stats(self, token: str) -> Dict[str, Any]:
        """Get detailed statistics for a specific token."""
        completed_cycles = self.calculate_completed_trade_cycles()
        trades_with_pnl = self.calculate_trade_pnl()
        
        # Filter cycles and trades for this token
        token_cycles = [c for c in completed_cycles if c['token'].upper() == token.upper()]
        token_trades = []
        for trade in trades_with_pnl:
            symbol = trade['symbol']
            trade_token = symbol.split('/')[0] if '/' in symbol else symbol
            if trade_token.upper() == token.upper():
                token_trades.append(trade)
        
        if not token_trades:
            return {
                'token': token.upper(),
                'total_trades': 0,
                'total_pnl': 0.0,
                'win_rate': 0.0,
                'message': f"No trading history found for {token.upper()}"
            }
        
        # Basic trade info (individual orders)
        total_trades = len(token_trades)
        buy_trades = len([t for t in token_trades if t['side'] == 'buy'])
        sell_trades = len([t for t in token_trades if t['side'] == 'sell'])
        total_volume = sum(t['value'] for t in token_trades)
        
        if not token_cycles:
            return {
                'token': token.upper(),
                'total_trades': total_trades,
                'buy_trades': buy_trades,
                'sell_trades': sell_trades,
                'total_volume': total_volume,
                'completed_trades': 0,  # No completed cycles
                'total_pnl': 0.0,
                'pnl_percentage': 0.0,
                'win_rate': 0.0,
                'message': f"{token.upper()} has open positions but no completed trade cycles yet"
            }
        
        # Performance calculations based on completed cycles
        total_pnl = sum(c['total_pnl'] for c in token_cycles)
        completed_volume = sum(c['sell_value'] for c in token_cycles)
        pnl_percentage = (total_pnl / completed_volume * 100) if completed_volume > 0 else 0.0
        
        # Win/loss analysis (by trade cycles)
        wins = [c['total_pnl'] for c in token_cycles if c['total_pnl'] > 0]
        losses = [abs(c['total_pnl']) for c in token_cycles if c['total_pnl'] < 0]
        
        total_wins = len(wins)
        total_losses = len(losses)
        completed_count = len(token_cycles)  # Number of completed cycles
        
        win_rate = (total_wins / completed_count * 100) if completed_count > 0 else 0
        avg_win = sum(wins) / total_wins if wins else 0
        avg_loss = sum(losses) / total_losses if losses else 0
        largest_win = max(wins) if wins else 0
        largest_loss = max(losses) if losses else 0
        
        total_profit = sum(wins) if wins else 0
        total_loss_amount = sum(losses) if losses else 0
        profit_factor = (total_profit / total_loss_amount) if total_loss_amount > 0 else float('inf') if total_profit > 0 else 0
        expectancy = avg_win * (win_rate/100) - avg_loss * ((100-win_rate)/100)
        
        # Recent trades (still show individual orders for detail)
        recent_trades = token_trades[-5:]  # Last 5 individual orders
        
        return {
            'token': token.upper(),
            'total_trades': total_trades,  # Total individual orders
            'completed_trades': completed_count,  # Number of completed cycles
            'buy_trades': buy_trades,
            'sell_trades': sell_trades,
            'total_volume': total_volume,
            'completed_volume': completed_volume,
            'total_pnl': total_pnl,
            'pnl_percentage': pnl_percentage,
            'win_rate': win_rate,
            'profit_factor': profit_factor,
            'avg_win': avg_win,
            'avg_loss': avg_loss,
            'largest_win': largest_win,
            'largest_loss': largest_loss,
            'expectancy': expectancy,
            'total_wins': total_wins,
            'total_losses': total_losses,
            'recent_trades': recent_trades,
            'cycles': token_cycles  # Include cycle details
        }

    def _update_period_stats(self):
        """Update daily, weekly, and monthly stats."""
        # Recalculate all period stats from completed trade cycles
        completed_cycles = self.calculate_completed_trade_cycles()
        
        # Reset period stats
        self.data['daily_stats'] = {}
        self.data['weekly_stats'] = {}
        self.data['monthly_stats'] = {}
        
        # Group completed cycles by periods and calculate stats
        for cycle in completed_cycles:
            timestamp = cycle['cycle_end']  # Use cycle end time
            pnl = cycle['total_pnl']
            value = cycle['sell_value']  # Use sell value for volume
            
            try:
                trade_date = datetime.fromisoformat(timestamp)
                
                # Daily stats
                day_key = trade_date.strftime('%Y-%m-%d')
                if day_key not in self.data['daily_stats']:
                    self.data['daily_stats'][day_key] = {'trades': 0, 'pnl': 0.0, 'volume': 0.0}
                
                self.data['daily_stats'][day_key]['trades'] += 1  # Count as 1 completed cycle
                self.data['daily_stats'][day_key]['pnl'] += pnl
                self.data['daily_stats'][day_key]['volume'] += value
                
                # Weekly stats (ISO week)
                week_key = trade_date.strftime('%Y-W%U')
                if week_key not in self.data['weekly_stats']:
                    self.data['weekly_stats'][week_key] = {'trades': 0, 'pnl': 0.0, 'volume': 0.0}
                
                self.data['weekly_stats'][week_key]['trades'] += 1  # Count as 1 completed cycle
                self.data['weekly_stats'][week_key]['pnl'] += pnl
                self.data['weekly_stats'][week_key]['volume'] += value
                
                # Monthly stats
                month_key = trade_date.strftime('%Y-%m')
                if month_key not in self.data['monthly_stats']:
                    self.data['monthly_stats'][month_key] = {'trades': 0, 'pnl': 0.0, 'volume': 0.0}
                
                self.data['monthly_stats'][month_key]['trades'] += 1  # Count as 1 completed cycle
                self.data['monthly_stats'][month_key]['pnl'] += pnl
                self.data['monthly_stats'][month_key]['volume'] += value
                
            except (ValueError, TypeError):
                # Skip cycles with invalid timestamps
                continue
        
        # Calculate percentages for all periods
        for period_data in [self.data['daily_stats'], self.data['weekly_stats'], self.data['monthly_stats']]:
            for period_key, stats in period_data.items():
                if stats['volume'] > 0:
                    stats['pnl_pct'] = (stats['pnl'] / stats['volume']) * 100
                else:
                    stats['pnl_pct'] = 0.0

    def get_daily_stats(self, limit: int = 10) -> List[Dict[str, Any]]:
        """Get daily performance stats for the last N days (including days with no trades)."""
        # Ensure period stats are up to date
        self._update_period_stats()
        
        # Generate the last N calendar days
        daily_stats = []
        today = datetime.now().date()
        
        for i in range(limit):
            # Calculate the date (going backwards from today)
            target_date = today - timedelta(days=i)
            date_str = target_date.strftime('%Y-%m-%d')
            date_formatted = target_date.strftime('%m/%d')
            
            # Check if we have stats for this date
            if date_str in self.data['daily_stats']:
                stats = self.data['daily_stats'][date_str]
                daily_stats.append({
                    'date': date_str,
                    'date_formatted': date_formatted,
                    'trades': stats['trades'],
                    'pnl': stats['pnl'],
                    'pnl_pct': stats['pnl_pct'],
                    'volume': stats['volume'],
                    'has_trades': True
                })
            else:
                # No trades on this date
                daily_stats.append({
                    'date': date_str,
                    'date_formatted': date_formatted,
                    'trades': 0,
                    'pnl': 0.0,
                    'pnl_pct': 0.0,
                    'volume': 0.0,
                    'has_trades': False
                })
        
        return daily_stats

    def get_weekly_stats(self, limit: int = 10) -> List[Dict[str, Any]]:
        """Get weekly performance stats for the last N weeks (including weeks with no trades)."""
        # Ensure period stats are up to date
        self._update_period_stats()
        
        # Generate the last N calendar weeks
        weekly_stats = []
        today = datetime.now().date()
        
        for i in range(limit):
            # Calculate the start of the week (going backwards from today)
            # Find the Monday of the target week
            days_since_monday = today.weekday()
            current_monday = today - timedelta(days=days_since_monday)
            target_monday = current_monday - timedelta(weeks=i)
            target_sunday = target_monday + timedelta(days=6)
            
            # Generate week string (YYYY-WXX format)
            week_str = target_monday.strftime('%Y-W%U')
            week_formatted = f"{target_monday.strftime('%m/%d')}-{target_sunday.strftime('%m/%d')}"
            
            # Check if we have stats for this week
            if week_str in self.data['weekly_stats']:
                stats = self.data['weekly_stats'][week_str]
                weekly_stats.append({
                    'week': week_str,
                    'week_formatted': week_formatted,
                    'trades': stats['trades'],
                    'pnl': stats['pnl'],
                    'pnl_pct': stats['pnl_pct'],
                    'volume': stats['volume'],
                    'has_trades': True
                })
            else:
                # No trades in this week
                weekly_stats.append({
                    'week': week_str,
                    'week_formatted': week_formatted,
                    'trades': 0,
                    'pnl': 0.0,
                    'pnl_pct': 0.0,
                    'volume': 0.0,
                    'has_trades': False
                })
        
        return weekly_stats

    def get_monthly_stats(self, limit: int = 10) -> List[Dict[str, Any]]:
        """Get monthly performance stats for the last N months (including months with no trades)."""
        # Ensure period stats are up to date
        self._update_period_stats()
        
        # Generate the last N calendar months
        monthly_stats = []
        today = datetime.now().date()
        current_month = today.replace(day=1)  # First day of current month
        
        for i in range(limit):
            # Calculate the target month (going backwards from current month)
            if i == 0:
                target_month = current_month
            else:
                # Go back i months
                year = current_month.year
                month = current_month.month - i
                
                # Handle year rollover
                while month <= 0:
                    month += 12
                    year -= 1
                
                target_month = datetime(year, month, 1).date()
            
            # Generate month string (YYYY-MM format)
            month_str = target_month.strftime('%Y-%m')
            month_formatted = target_month.strftime('%b %Y')
            
            # Check if we have stats for this month
            if month_str in self.data['monthly_stats']:
                stats = self.data['monthly_stats'][month_str]
                monthly_stats.append({
                    'month': month_str,
                    'month_formatted': month_formatted,
                    'trades': stats['trades'],
                    'pnl': stats['pnl'],
                    'pnl_pct': stats['pnl_pct'],
                    'volume': stats['volume'],
                    'has_trades': True
                })
            else:
                # No trades in this month
                monthly_stats.append({
                    'month': month_str,
                    'month_formatted': month_formatted,
                    'trades': 0,
                    'pnl': 0.0,
                    'pnl_pct': 0.0,
                    'volume': 0.0,
                    'has_trades': False
                })
        
        return monthly_stats

    def record_deposit(self, amount: float, timestamp: str = None, deposit_id: str = None):
        """Record a deposit to adjust P&L calculations."""
        if timestamp is None:
            timestamp = datetime.now().isoformat()
            
        deposit_record = {
            'timestamp': timestamp,
            'amount': amount,
            'type': 'deposit',
            'id': deposit_id,
            'description': f'Deposit of ${amount:.2f}'
        }
        
        # Initialize balance_adjustments if not present
        if 'balance_adjustments' not in self.data:
            self.data['balance_adjustments'] = []
        
        self.data['balance_adjustments'].append(deposit_record)
        
        # Adjust initial balance to maintain P&L accuracy
        self.data['initial_balance'] += amount
        
        logger.info(f"💰 Recorded deposit: ${amount:.2f} (adjusted initial balance to ${self.data['initial_balance']:.2f})")
        self._save_stats()
    
    def record_withdrawal(self, amount: float, timestamp: str = None, withdrawal_id: str = None):
        """Record a withdrawal to adjust P&L calculations."""
        if timestamp is None:
            timestamp = datetime.now().isoformat()
            
        withdrawal_record = {
            'timestamp': timestamp,
            'amount': -amount,  # Negative for withdrawal
            'type': 'withdrawal',
            'id': withdrawal_id,
            'description': f'Withdrawal of ${amount:.2f}'
        }
        
        # Initialize balance_adjustments if not present
        if 'balance_adjustments' not in self.data:
            self.data['balance_adjustments'] = []
        
        self.data['balance_adjustments'].append(withdrawal_record)
        
        # Adjust initial balance to maintain P&L accuracy
        self.data['initial_balance'] -= amount
        
        logger.info(f"💸 Recorded withdrawal: ${amount:.2f} (adjusted initial balance to ${self.data['initial_balance']:.2f})")
        self._save_stats()
    
    def get_balance_adjustments_summary(self) -> Dict[str, Any]:
        """Get summary of all balance adjustments (deposits/withdrawals)."""
        adjustments = self.data.get('balance_adjustments', [])
        
        if not adjustments:
            return {
                'total_deposits': 0.0,
                'total_withdrawals': 0.0,
                'net_adjustment': 0.0,
                'adjustment_count': 0,
                'last_adjustment': None
            }
        
        deposits = [adj for adj in adjustments if adj['type'] == 'deposit']
        withdrawals = [adj for adj in adjustments if adj['type'] == 'withdrawal']
        
        total_deposits = sum(adj['amount'] for adj in deposits)
        total_withdrawals = sum(abs(adj['amount']) for adj in withdrawals)
        net_adjustment = total_deposits - total_withdrawals
        
        # Find most recent adjustment
        latest_adjustment = max(adjustments, key=lambda x: x['timestamp']) if adjustments else None
        
        return {
            'total_deposits': total_deposits,
            'total_withdrawals': total_withdrawals,
            'net_adjustment': net_adjustment,
            'adjustment_count': len(adjustments),
            'last_adjustment': latest_adjustment['timestamp'] if latest_adjustment else None
        }

    def calculate_completed_trade_cycles(self) -> List[Dict[str, Any]]:
        """Calculate completed trade cycles (full position open to close) using FIFO method."""
        completed_cycles = []
        positions = {}  # Track open positions by symbol
        
        for trade in self.data['trades']:
            symbol = trade['symbol']
            side = trade['side']
            amount = trade['amount']
            price = trade['price']
            timestamp = trade['timestamp']
            
            if symbol not in positions:
                positions[symbol] = {
                    'amount': 0, 
                    'avg_price': 0, 
                    'total_cost': 0,
                    'cycle_trades': [],  # Track all trades in current cycle
                    'cycle_start': None
                }
            
            pos = positions[symbol]
            
            if side == 'buy':
                # Add to position
                was_zero = pos['amount'] == 0
                pos['total_cost'] += amount * price
                pos['amount'] += amount
                pos['avg_price'] = pos['total_cost'] / pos['amount'] if pos['amount'] > 0 else 0
                
                # If starting a new cycle, record start time
                if was_zero:
                    pos['cycle_start'] = timestamp
                    pos['cycle_trades'] = []
                
                # Add trade to current cycle
                pos['cycle_trades'].append({
                    'side': side,
                    'amount': amount,
                    'price': price,
                    'timestamp': timestamp,
                    'value': amount * price
                })
                
            elif side == 'sell':
                # Reduce position and calculate PnL
                if pos['amount'] > 0:
                    sold_amount = min(amount, pos['amount'])
                    pnl = sold_amount * (price - pos['avg_price'])
                    
                    # Add trade to current cycle
                    pos['cycle_trades'].append({
                        'side': side,
                        'amount': sold_amount,
                        'price': price,
                        'timestamp': timestamp,
                        'value': sold_amount * price,
                        'pnl': pnl
                    })
                    
                    # Update position
                    pos['amount'] -= sold_amount
                    if pos['amount'] > 0:
                        pos['total_cost'] = pos['amount'] * pos['avg_price']
                    else:
                        # Position fully closed - complete trade cycle
                        pos['total_cost'] = 0
                        pos['avg_price'] = 0
                        
                        # Calculate cycle totals
                        cycle_buys = [t for t in pos['cycle_trades'] if t['side'] == 'buy']
                        cycle_sells = [t for t in pos['cycle_trades'] if t['side'] == 'sell']
                        
                        total_buy_value = sum(t['value'] for t in cycle_buys)
                        total_sell_value = sum(t['value'] for t in cycle_sells)
                        total_cycle_pnl = sum(t.get('pnl', 0) for t in cycle_sells)
                        
                        # Create completed cycle record
                        completed_cycle = {
                            'symbol': symbol,
                            'token': symbol.split('/')[0] if '/' in symbol else symbol,
                            'cycle_start': pos['cycle_start'],
                            'cycle_end': timestamp,
                            'buy_orders': len(cycle_buys),
                            'sell_orders': len(cycle_sells),
                            'total_orders': len(pos['cycle_trades']),
                            'total_amount': sum(t['amount'] for t in cycle_buys),
                            'avg_entry_price': total_buy_value / sum(t['amount'] for t in cycle_buys) if cycle_buys else 0,
                            'avg_exit_price': total_sell_value / sum(t['amount'] for t in cycle_sells) if cycle_sells else 0,
                            'total_pnl': total_cycle_pnl,
                            'buy_value': total_buy_value,
                            'sell_value': total_sell_value,
                            'cycle_trades': pos['cycle_trades'].copy()
                        }
                        
                        completed_cycles.append(completed_cycle)
                        
                        # Reset for next cycle
                        pos['cycle_trades'] = []
                        pos['cycle_start'] = None
        
        return completed_cycles 